--^=====================================^--
--)-----[ AW's EVERYMOD UTILITIES ]-----(--
--V=====================================V--
-->------> UTILITY MATH FUNCTIONS <-----<--

--|=============================================================|
--| A file full of utility functions for dealing with numbers.  |
--| Mostly my own, but with a couple thrown in from other		|
--| modders.													|
--|=============================================================|
--| "Copyright" 2/18/2016, AgressiveWimp Studios Inc.			|
--| (I kid, I kid. It's for all use, though I don't know who	|
--| will bother to use it.)										|
--|=============================================================|

-- From: JFD
-- Description: Returns a random integer between "lower" and "upper" inclusive
function GetRandom(lower, upper)
    return Game.Rand((upper + 1) - lower, "") + lower
end

-- From: Self, based on JFD
-- Description: Returns a random integer between "lower" and "upper" eXclusive
function GetRandomX(lower, upper)
	lower = lower + 1
	upper = upper - 1
	return Game.Rand((upper + 1) - lower, "") + lower
end

-- From: Self
-- Description: Shorthand way to change a number "input" to another predetermined number "alt" if it happens to be the undesirable number "constraint"
function ifx(input, constraint, alt)
	temp = input
	if temp == constraint then temp = alt end
	return temp
end

-- From: Self
-- Description: Shorthand way to determine the greater of two numbers
function xmax(a,b)
	if a > b then return a end
	return b
end

-- From: Self
-- Description: Shorthand way to determine the lesser of two numbers
function xmin(a,b)
	if a < b then return a end
	return b
end

-- From: Self
function iPart(x)
	return math.floor(x)
end

-- From: Self
function fPart(x)
	return x - iPart(x)
end

-- From: Self
function isInt(x)
	if (x - iPart(x) == 0) then return true end
	return false
end

-- From: Self
function RollTheDice(numerator, denominator)
	local iRoll = GetRandom(1, denominator)
	if (numerator <= iRoll) then return true end
	return false
end

-- From: Self
function PercentChance(percent)
	local bRoll
	if isInt(percent) then
		bRoll = RollTheDice(percent, 100)
	else
		local numerator = percent
		local denominator = 100
		while not isInt(numerator) do
			numerator = numerator * 10
			denominator = denominator * 10
		end
		bRoll = RollTheDice(numerator, denominator)
	end
	return bRoll
end

-- From: UltimatePotato
function toBits(num)
    -- returns a table of bits, least significant first.
	t={} -- will contain the bits
    while num>0 do
        local rest=math.fmod(num,2)
        t[#t+1]=rest
        num=(num-rest)/2
    end
    return t
end
-- AW NOTE: This will return an array of ones and zeroes, with increasing key values representing an increasing power of two
-- e.g. toBits(16) = {
--			1:	0
--			2:	0
--			3:	0
--			4:	0
--			5:	1
--		}
-- Notice this is little-endian
-- COmpare with my function bitConv if that suits your purpose better

-- From: Self
-- Returns a triangular number for n; that is, the resultant of sum(k=1)(n) of k
-- also expressed 1 + 2 + 3 + 4 + ... + n
function triNum(n)
	return ((n*(n+1))/2)
end

-- From: Self
-- Made this when I realized UltimatePotato's toBits didn't do what I thought
function bitConv(x)
	bitTable = {}
	-- PART ONE --
	-- find at what exponent "n", 2^n >= x
	if (x >= 0) then
		exponent = 0
		while ((2^exponent) < x) do
			exponent = exponent + 1
		end
		-- PART TWO --
		-- Now that exponent "n", is found, raise 2^n
		-- and subtract it from x as long as x >= 2^n
		-- after every iteration, subtract 1 from n
		-- (or in other words, do (2^n)/2 )
		-- until eventually x = 0
		upper = 2^exponent
		temp = x
		while (upper > 0) do
			if (temp >= upper) then
				table.insert(bitTable, math.floor(upper))
				temp = temp - upper
			end
			upper = upper / 2
		end
	end
	return bitTable
end
-- this will return a list of all the powers of two which, when added together, equal x
-- e.g. bitConv(42) = {
--		1:	32
--		2:	8
--		3:	2
--	}
-- Compare UltimatePotato's toBits function if that serves your purpose better
